@@ -363,14 +363,14 @@ class NF_visualization(HasTraits):
                        colormap='Blues', 
                        scale=2.5, clim=[0.2, 1.0], cycle_moment=0,
                        axis_labels=True, abs_scale=False,
-                       reset_view=False, clearfig=True,
+                       reset_view=False, clearfig=True, which_field='E',
                        **kwargs):
         self.sim = sim
         self.field_index = field_index
         self.scale = scale
         self.Nframes = Nframes
         
-        NF = tools.get_field_as_list_by_fieldindex(sim, field_index).T
+        NF = tools.get_field_as_list_by_fieldindex(sim, field_index, which_field).T
         x,y,z = NF[0:3].real
         
         self. maxdist = max([max(x)-min(x), max(y)-min(y)])
