@@ -13,6 +13,8 @@ from pyGDM2 import tools
 from pyGDM2 import structures
 from pyGDM2 import materials
 from pyGDM2 import fields
+from pyGDM2 import propagators
+from pyGDM2 import propagators_2D
 from pyGDM2 import core
 from pyGDM2 import visu
 
@@ -60,9 +62,9 @@ class QThreadRunSim(QtCore.QThread):
     def set_sim(self, sim):
         self.sim = sim
         
-    def set_method(self, setupmethod, method):
+    def set_method(self, method, calcH):
         self.method = method
-        self.setupmethod = setupmethod
+        self.calcH = calcH
 
     def cancel(self):
         self.cont = False
@@ -76,8 +78,9 @@ class QThreadRunSim(QtCore.QThread):
         self.running = True
         core.scatter(self.sim, 
                      method=self.method, 
-                     matrix_setup=self.setupmethod, 
-                     callback=self.emit_status, verbose=False)
+                     calc_H=self.calcH, 
+                     callback=self.emit_status, 
+                     verbose=False)
         self.sigFinishSimRun.emit(self.sim)
 
         
@@ -165,6 +168,8 @@ class MyApplication(QtWidgets.QMainWindow):
         self.ui.listWidgetAvailableFieldConfigs.itemSelectionChanged.connect(self.on_click_select_sim_config)
         self.ui.lineEditAnimateIntFieldScale.textChanged.connect(self.on_click_select_sim_config)
         self.ui.horizontalSliderPlotMomentOpticalCycleIntField.sliderReleased.connect(self.on_click_select_sim_config)
+        self.ui.radioButtonE.toggled.connect(self.on_click_select_sim_config)
+        self.ui.radioButtonH.toggled.connect(self.on_click_select_sim_config)
         
         self.simRunThread = QThreadRunSim()
         self.sigCancelSim.connect(self.simRunThread.cancel)
@@ -203,6 +208,9 @@ class MyApplication(QtWidgets.QMainWindow):
 #         populate items
 # =============================================================================
         ## --- geometry tab
+        self.ui.comboBoxDyads.clear()
+        self.ui.comboBoxDyads.addItems(['3D', '2D'])
+        
         self.ui.comboBoxMesh.clear()
         self.ui.comboBoxMesh.addItems(['cube', 'hex'])
         
@@ -280,41 +288,51 @@ class MyApplication(QtWidgets.QMainWindow):
             self.load_struct(filename + ".pkl.pygdmstruct")
         except:
             warnings.warn("No saved structure found with the simulation. Loading geometry from sim as generic model.")
-            self.load_struct(self.sim.struct)
+            self.load_struct(self.sim)
             
 
     def save_struct(self, filename):
         pickle.dump(dict(geolist=self.geolist, 
                              materialslist=self.materialslist,
                              subgeotypelist=self.subgeotypelist, 
-                             struct_full=self.struct_full
+                             struct_full=self.struct_full,
+                             dyads=self.dyads
                              ),
                         open(filename, "wb"))
     
     def load_struct(self, filename):
-        if type(filename) == structures.struct:
-            struct = filename
+        if type(filename) == core.simulation:
+            sim = filename
+            struct = sim.struct
             self.on_click_clear_geo(show_dialog=False)
             self.geolist = [struct.geometry]
             self.materialslist = [struct.material]
             self.subgeotypelist = [dummystruct()]
-        else:
+            self.dyads = sim.dyads
+        else:   # no simulation --> got filename to load list of geometries
             struct_dict = pickle.load(open(filename, "rb"))
             self.on_click_clear_geo(show_dialog=False)
             struct = struct_dict['struct_full']
             self.materialslist = struct_dict['materialslist']
             self.subgeotypelist = struct_dict['subgeotypelist']
             self.geolist = struct_dict['geolist']
+            self.dyads = struct_dict['dyads']
         
-        if struct.normalization == 1:
+        ## check mesh type
+        mesh = tools.get_mesh_from_geometry(self.geolist[0])
+        if mesh.lower() == 'cube':
             self.ui.comboBoxMesh.setCurrentIndex(0)
         else:
             self.ui.comboBoxMesh.setCurrentIndex(1)
-            
-        self.ui.lineEditN1.setText(str(struct.n1))
-        self.ui.lineEditN2.setText(str(struct.n2))
-        self.ui.lineEditN3.setText(str(struct.n3))
-        self.ui.lineEditSpacing.setText(str(float(struct.spacing)))
+        
+        self.dyads.getEnvironmentIndices(500, self.geolist[0])
+        self.ui.lineEditN1.setText(str(self.dyads.n1))
+        self.dyads.getEnvironmentIndices(500, self.geolist[0])
+        self.ui.lineEditN2.setText(str(self.dyads.n2))
+        self.dyads.getEnvironmentIndices(500, self.geolist[0])
+        self.ui.lineEditN3.setText(str(self.dyads.n3))
+        
+        self.ui.lineEditSpacing.setText(str(float(self.dyads.spacing)))
         self.ui.lineEditStep.setText(str(float(struct.step)))
         
         ## list of constituents
@@ -467,6 +485,11 @@ class MyApplication(QtWidgets.QMainWindow):
         self.n3 = complex(self.ui.lineEditN3.text())
         self.spacing = float(self.ui.lineEditSpacing.text())
         
+        if self.ui.comboBoxDyads.currentText().lower() == '3d':
+            self.dyads = propagators.DyadsQuasistatic123(self.n1, self.n2, self.n2, self.spacing)
+        else:   # 2D
+            self.dyads = propagators_2D.DyadsQuasistatic2D123(self.n1, self.n2, self.n2, self.spacing)
+        
     def on_change_mesh_step(self):
         if self.ui.listWidgetStructures.count() >= 1:
             QtWidgets.QMessageBox.question(self, 'warning', 
@@ -594,11 +617,8 @@ class MyApplication(QtWidgets.QMainWindow):
             mat_all = np.concatenate(self.materialslist)
         
             self.struct_full = structures.struct(self.step, geo_all, mat_all, 
-                                            n1=self.n1, n2=self.n2, n3=self.n3,
-                                            spacing=self.spacing,
-                                            normalization=structures.get_normalization(self.mesh),
-                                            auto_shift_structure=False)
-            # self.struct_full.geometry = geo_all
+                                                 auto_shift_structure=False)
+            
             ## --- visu
             mat_labels = self.ui.checkBoxGeoMaterialLabels.checkState()
             self.ui.mayaviWidgetGeo.visualization.plot_struct(self.struct_full, 
@@ -761,7 +781,7 @@ class MyApplication(QtWidgets.QMainWindow):
         i_row = 0
         for i, kwstr in enumerate(kwarg_str_list):
             kwstr = kwstr.split('=')[0]
-            if kwstr not in ['wavelength', 'struct', 'returnField']:
+            if kwstr not in ['wavelength', 'pos', 'env_dict', 'returnField', '**kwargs', '']:
                 item = QtWidgets.QTableWidgetItem()
                 item.setText(kwstr)
                 item.setFlags(QtCore.Qt.ItemIsEnabled)
@@ -790,43 +810,46 @@ class MyApplication(QtWidgets.QMainWindow):
         ## --- setup field generator config
         kwargs = dict()
         for row in range(self.ui.tableWidgetFieldParams.rowCount()):
-            key = self.ui.tableWidgetFieldParams.item(row, 0).text()
-            val1 = self.ui.tableWidgetFieldParams.item(row, 1).text()
-            val2 = self.ui.tableWidgetFieldParams.item(row, 2).text()
-            N = int(self.ui.tableWidgetFieldParams.item(row, 3).text())
-            
-            default = signature(self.field_gen).parameters[key].default
-            # print(row, self.ui.tableWidgetFieldParams.rowCount(), type(default))
-            if type(default) == bool:
-                val = False if (val1.lower()=="false" or val1==0) else True
-            elif type(default) == str:
-                val = str(val1)
-            elif default != inspect._empty and default is not None:
-                val1 = type(default)(val1)
-                val2 = type(default)(val2)
-                val = np.linspace(val1, val2, N)
-            elif default is None and val1.lower()=='none':
-                val = None
-            else:
-                try:
-                    val1 = float(val1)
-                    if N>1:
-                        val2 = float(val2)
-                        val = np.linspace(val1, val2, N)
-                    else:
-                        val = val1
-                except ValueError:
-                    val = str(val1)  # ignore second value if not a number
-            # print(val, default, val is not default, val != default)
-            if val is not default:
-                kwargs[key] = val
+            if self.ui.tableWidgetFieldParams.item(row, 0) is not None:
+                key = self.ui.tableWidgetFieldParams.item(row, 0).text()
+                val1 = self.ui.tableWidgetFieldParams.item(row, 1).text()
+                val2 = self.ui.tableWidgetFieldParams.item(row, 2).text()
+                N = int(self.ui.tableWidgetFieldParams.item(row, 3).text())
+                
+                
+                default = signature(self.field_gen).parameters[key].default
+                # print(row, self.ui.tableWidgetFieldParams.rowCount(), type(default))
+                if type(default) == bool:
+                    val = False if (val1.lower()=="false" or val1==0) else True
+                elif type(default) == str:
+                    val = str(val1)
+                elif default != inspect._empty and default is not None:
+                    val1 = type(default)(val1)
+                    val2 = type(default)(val2)
+                    val = np.linspace(val1, val2, N)
+                elif default is None and val1.lower()=='none':
+                    val = None
+                else:
+                    try:
+                        val1 = float(val1)
+                        if N>1:
+                            val2 = float(val2)
+                            val = np.linspace(val1, val2, N)
+                        else:
+                            val = val1
+                    except ValueError:
+                        val = str(val1)  # ignore second value if not a number
+                # print(val, default, val is not default, val != default)
+                if val is not default:
+                    kwargs[key] = val
         # print (kwargs)
         wl0 = float(self.ui.lineEditWl0.text())
         wl1 = float(self.ui.lineEditWl1.text())
         Nwl = int(self.ui.lineEditNwl.text())
         self.wavelengths = np.linspace(wl0, wl1, Nwl)
         self.efield = fields.efield(self.field_gen, self.wavelengths, kwargs)
-        self.sim = core.simulation(self.struct_full, self.efield)
+        
+        self.sim = core.simulation(self.struct_full, self.efield, self.dyads)
         
         
     def clear_simulation(self):
@@ -851,9 +874,9 @@ class MyApplication(QtWidgets.QMainWindow):
             self.gen_sim_instance()
             
             self.simRunThread.set_sim(self.sim)
-            setupmethod = self.ui.comboBoxSetupMethod.currentText()
             method = self.ui.comboBoxInversionMethod.currentText()
-            self.simRunThread.set_method(setupmethod=setupmethod, method=method)
+            calcH = self.ui.checkBoxCalcH.checkState()
+            self.simRunThread.set_method(method=method, calcH=calcH)
             self.simRunThread.start()
             
             msg = tools.print_sim_info(self.sim, prnt=False)
@@ -914,6 +937,10 @@ class MyApplication(QtWidgets.QMainWindow):
             
             self.ui.plainTextEditSelectedConfig.clear()
             self.ui.plainTextEditSelectedConfig.insertPlainText(msg)
+            if self.ui.radioButtonE.isChecked():
+                plot_field = 'E'
+            else:
+                plot_field = 'H'
 
             ## --- load internal fields into visualization
             search_kwargs = copy.deepcopy(self.sim.efield.kwargs_permutations[i_conf])
@@ -925,19 +952,19 @@ class MyApplication(QtWidgets.QMainWindow):
             self.ui.widgetMPL_2dNF_XY.canvas.fig.clear()
             ax = self.ui.widgetMPL_2dNF_XY.canvas.ax = self.ui.widgetMPL_2dNF_XY.canvas.fig.add_subplot(111)
             visu.vectorfield_by_fieldindex(self.sim, f_idx, ax=ax, tit='XY',
-                                           projection='XY', show=False)
+                                           projection='XY', which_field=plot_field, show=False)
             self.ui.widgetMPL_2dNF_XY.canvas.draw()
             
             self.ui.widgetMPL_2dNF_XZ.canvas.fig.clear()
             ax = self.ui.widgetMPL_2dNF_XZ.canvas.ax = self.ui.widgetMPL_2dNF_XZ.canvas.fig.add_subplot(111)
             visu.vectorfield_by_fieldindex(self.sim, f_idx, ax=ax, tit='XZ', 
-                                           projection='XZ', show=False)
+                                           projection='XZ', which_field=plot_field, show=False)
             self.ui.widgetMPL_2dNF_XZ.canvas.draw()
             
             self.ui.widgetMPL_2dNF_YZ.canvas.fig.clear()
             ax = self.ui.widgetMPL_2dNF_YZ.canvas.ax = self.ui.widgetMPL_2dNF_YZ.canvas.fig.add_subplot(111)
             visu.vectorfield_by_fieldindex(self.sim, f_idx, ax=ax, tit='YZ',
-                                           projection='YZ', show=False)
+                                           projection='YZ', which_field=plot_field, show=False)
             self.ui.widgetMPL_2dNF_YZ.canvas.draw()
             
             
@@ -951,7 +978,7 @@ class MyApplication(QtWidgets.QMainWindow):
             #                              clearfig=False, reset_view=False)
             self.ui.mayaviWidgetNF.visualization.init_field_dat(self.sim, field_index=f_idx, 
                                         scale=3.5*scale, cycle_moment=cycle_moment, 
-                                        clim=[-0.2, 0.9], mode='arrow',
+                                        clim=[-0.2, 0.9], mode='arrow', which_field=plot_field,
                                         clearfig=False, reset_view=False)
             
             
@@ -1048,7 +1075,9 @@ class MyApplication(QtWidgets.QMainWindow):
                 del kwargs['y']
                 del kwargs['z']
             wl, spec = tools.calculate_spectrum(self.sim, i_conf, 
-                                            self.spectra_func["func"], **kwargs)
+                                                self.spectra_func["func"], 
+                                                callback=self.on_spectrum_status,
+                                                **kwargs)
             
             if self.spectra_func['postprocess'] is not None:
                 spec = self.spectra_func['postprocess'](spec)
@@ -1056,6 +1085,14 @@ class MyApplication(QtWidgets.QMainWindow):
             self.do_plot_spectrum(wl, spec, self.spectra_func)
             self.spec = spec
     
+    
+    def on_spectrum_status(self, status_dict):
+        # dict-keys: 'spectrum' (values), 'wavelengths'
+        percent_done = int(100*len(status_dict['wavelengths']) / len(self.sim.efield.wavelengths))
+        self.ui.progressBarRasterscan.setValue(percent_done)
+        return(True)   # continue: True
+    
+    
     def do_plot_spectrum(self, wl, spec, f_dict):
         canvas = self.ui.widgetMPLspectrum.canvas
         
@@ -1272,20 +1309,18 @@ class MyApplication(QtWidgets.QMainWindow):
                 ## draw structure contour
                 if self.ui.checkBoxMappingGeoContour.checkState():
                     visu.structure_contour(self.sim, color='w', 
-                               input_mesh=self.ui.comboBoxMesh.currentText(),
                                ax=ax, show=0, projection=projection)
                     visu.structure_contour(self.sim, color='k', dashes=[2,2],
-                               input_mesh=self.ui.comboBoxMesh.currentText(),
                                ax=ax, show=0, projection=projection)
                 
                 for k in idx_1D:
                     if k not in projection:
-                        title += "{}={:.1f}nm ".format(k, float(mapping[0][idx_1D[k]]))
+                        title += "{}={:.1f}nm ".format(k, float(np.real(mapping[0][idx_1D[k]])))
 
                 self.mapping_scan_title = title
                 if self.mapping_save_dat == []:
-                    self.mapping_save_dat.append(mapping.T[:3].astype(float))
-                self.mapping_save_dat.append(mapping.T[3].astype(float)[None, :])
+                    self.mapping_save_dat.append(np.real(mapping.T[:3]).astype(float))
+                self.mapping_save_dat.append(np.real(mapping.T[3]).astype(float)[None, :])
                 
                 ax.set_title(f_dict['results_titles'][i] + ' - ' + title)
                 ax.set_xlabel("{} (nm)".format(projection[0]))
@@ -1305,7 +1340,7 @@ class MyApplication(QtWidgets.QMainWindow):
         
         canvas = self.ui.widgetMPLmapping.canvas
         canvas.fig.clear()
-        for i, I in enumerate([Is, It]):
+        for i, I in enumerate([Is]):
             ax = canvas.ax = canvas.fig.add_subplot(f_dict['plot_layout'][0], 
                                                     f_dict['plot_layout'][1],
                                                     i+1, polar=True)
@@ -1467,8 +1502,8 @@ class MyApplication(QtWidgets.QMainWindow):
                     ax.legend()
                     
                 if self.rasterscan_save_dat == []:
-                    self.rasterscan_save_dat.append(coords.T[idx_1D].astype(float))
-                self.rasterscan_save_dat.append(mapping.astype(float))
+                    self.rasterscan_save_dat.append(np.real(coords.T[idx_1D]).astype(float))
+                self.rasterscan_save_dat.append(np.real(mapping).astype(float))
                 
             ## --- 2D data
             if len(projection) == 2:
@@ -1479,10 +1514,8 @@ class MyApplication(QtWidgets.QMainWindow):
                 ## draw structure contour
                 if self.ui.checkBoxRasterscanGeoContour.checkState():
                     visu.structure_contour(self.sim, color='w', 
-                               input_mesh=self.ui.comboBoxMesh.currentText(),
                                ax=ax, show=0)
                     visu.structure_contour(self.sim, color='k', dashes=[2,2],
-                               input_mesh=self.ui.comboBoxMesh.currentText(),
                                ax=ax, show=0)
                 
                 ax.set_title(f_dict['results_titles'][i] + title)
@@ -1492,8 +1525,8 @@ class MyApplication(QtWidgets.QMainWindow):
                 ax.autoscale(tight=True)
                 
                 if self.rasterscan_save_dat == []:
-                    self.rasterscan_save_dat.append(coords.T.astype(float))
-                self.rasterscan_save_dat.append(mapping.astype(float)[None, :])
+                    self.rasterscan_save_dat.append(np.real(coords.T).astype(float))
+                self.rasterscan_save_dat.append(np.real(mapping).astype(float)[None, :])
                 
         canvas.fig.tight_layout()
         canvas.draw()
